<?php

namespace App\Http\Controllers\Transaction;

use App\Currency;
use App\Http\Controllers\Controller;
use App\InventoryOpeningBalance;
use App\InventoryOpeningBalanceItem;
use Auth;
use DB;
use Storage;
use Session;
use Carbon\Carbon;
use App\Services\Utils;
use App\Services\Slug;
use App\ItemName;
use App\PurchaseInvoice;
use App\PurchaseInvoiceItem;
use App\SaleInvoiceItem;
use Illuminate\Http\Request;
use App\WareHouseTrack;
use App\PurchaseSalesTransferStockTrack;

class InventoryOpeningBalanceController extends Controller
{
    protected $utils;
    protected $slug;

    public function __construct(Utils $utils, Slug $slug)
    {
        $this->utils = $utils;
        $this->slug = $slug;
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        if (!$user->can('opening-balances-for-inventory')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $inventory_opening_balances = PurchaseInvoice::withTrashed()->with('account','items.name')->where('location_id', $location_id)->Where('invoice_no', '=', 0)->orderBy('created_at','DESC')->paginate(100);

        return view('user.transactions.inventory_opening_balance.index', compact('inventory_opening_balances'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        if (!$user->can('opening-balances-for-inventory')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $item_names = ItemName::Where('location_id', $location_id)->get();
        $purchase_invoice = PurchaseInvoice::with('items.name', 'items.unit')->where('location_id', $location_id)->Where('invoice_no', 0)->first();
        // $purchase_invoice = PurchaseInvoice::Where('location_id', $location_id)->Where('invoice_no', 0)->first();
        $currency = Currency::Where('location_id', $location_id)->first();
        $warehouses = WareHouseTrack::with(['warehouse' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        return view('user.transactions.inventory_opening_balance.create',compact('currency','item_names', 'purchase_invoice', 'warehouses'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();
            $purchase_invoice = new PurchaseInvoice();
            $purchase_invoice->location_id = $location_id;
            $purchase_invoice->invoice_no = 0;
            $purchase_invoice->narrations = $request->narrations;
            $purchase_invoice->is_signed = 0;
            $purchase_invoice->save();

            foreach ($request->item_name_id as $key => $item_name_id) {
                $purchase_invoice_item = new PurchaseInvoiceItem();
                $purchase_invoice_item-> purchase_invoice_id = $purchase_invoice->id;
                $purchase_invoice_item-> item_name_id = $request->item_name_id[$key];
                $purchase_invoice_item-> item_unit_id = $request->item_unit[$key];
                $purchase_invoice_item-> description = $request->description[$key];
                // $purchase_invoice_item-> color = $request->color[$key];
                $purchase_invoice_item-> imei_serial_no = $request->imei_serial_no[$key];
                $purchase_invoice_item-> warehouse_id = $request->warehouse_id[$key];
                $purchase_invoice_item-> quantity = $request->quantity[$key];
                $purchase_invoice_item-> rate_per_quantity = $request->rate_per_quantity[$key];
                $purchase_invoice_item-> amount = $request->amount[$key];
                $purchase_invoice_item-> remarks = $request->remarks[$key];
                $purchase_invoice_item-> save();

                $purchase_sales_transfer_stock_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $request->imei_serial_no[$key])->Where('location_id', $location_id)->first();
                if ($purchase_sales_transfer_stock_track) {
                    return redirect()->back()->withErrors("IMEI Already Exist In This Location");
                }else{
                    $purchase_sales_transfer_stock_track = new PurchaseSalesTransferStockTrack();
                    $purchase_sales_transfer_stock_track-> location_id = $location_id;
                    $purchase_sales_transfer_stock_track-> warehouse_id = $request->warehouse_id[$key];
                    $purchase_sales_transfer_stock_track-> item_name_id = $request->item_name_id[$key];
                    $purchase_sales_transfer_stock_track-> item_unit_id = $request->item_unit[$key];
                    $purchase_sales_transfer_stock_track-> description = $request->description[$key];
                    // $purchase_sales_transfer_stock_track-> color = $request->color[$key];
                    $purchase_sales_transfer_stock_track-> imei_serial_no = $request->imei_serial_no[$key];
                    $purchase_sales_transfer_stock_track-> quantity = $request->quantity[$key];
                    $purchase_sales_transfer_stock_track-> rate_per_quantity = $request->rate_per_quantity[$key];
                    $purchase_sales_transfer_stock_track-> amount = $request->amount[$key];
                    $purchase_sales_transfer_stock_track-> save();
                }
            }
            DB::commit();
            Session::flash('alert-success','Inventory Opening Balance created successfully');
            return redirect('/transactions/inventory-opening-balances');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Display the specified resource.
     *
     * @param  \App\InventoryOpeningBalance  $inventoryOpeningBalance
     * @return \Illuminate\Http\Response
     */
    public function show(InventoryOpeningBalance $inventoryOpeningBalance)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  \App\InventoryOpeningBalance  $inventoryOpeningBalance
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $user = Auth::user();
        if (!$user->can('opening-balances-for-inventory')) {
            abort(403, 'Unauthorized action.');
        }

        $location_id = $this->getLocationId();
        $item_names = ItemName::Where('location_id', $location_id)->get();
        $purchase_invoice_item = PurchaseInvoiceItem::where('id', $id)->first();
        // $purchase_invoice = PurchaseInvoice::Where('location_id', $location_id)->Where('invoice_no', 0)->first();
        $currency = Currency::Where('location_id', $location_id)->first();
        $warehouses = WareHouseTrack::with(['warehouse' => function($q) use ($location_id) {
            $q->where('location_id', $location_id);}])->Where('user_id', auth()->user()->id)->get();

        return view('user.transactions.inventory_opening_balance.edit',compact('currency','item_names', 'purchase_invoice_item', 'warehouses'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\InventoryOpeningBalance  $inventoryOpeningBalance
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        DB::beginTransaction();

        try {
            $location_id = $this->getLocationId();

            // PurchaseInvoiceItem::where('purchase_invoice_id', $id)->forcedelete();
            $purchase_item = PurchaseInvoiceItem::with(['purchase_sale_stock' => function($q) use ($location_id) {
                $q->where('location_id', $location_id);}])->where('id', $id)->first();

            $purchase_item->purchase_sale_stock()->forcedelete();
            $purchase_item->forceDelete();

            $purchase_invoice_item = new PurchaseInvoiceItem();
            $purchase_invoice_item-> purchase_invoice_id = $purchase_item->purchase_invoice_id;
            $purchase_invoice_item-> item_name_id = $request->item_name_id;
            $purchase_invoice_item-> item_unit_id = $request->item_unit;
            $purchase_invoice_item-> description = $request->description;
            // $purchase_invoice_item-> color = $request->color;
            $purchase_invoice_item-> imei_serial_no = $request->imei_serial_no;
            $purchase_invoice_item-> warehouse_id = $request->warehouse_id;
            $purchase_invoice_item-> quantity = $request->quantity;
            $purchase_invoice_item-> rate_per_quantity = $request->rate_per_quantity;
            $purchase_invoice_item-> amount = $request->amount;
            $purchase_invoice_item-> remarks = $request->remarks;
            $purchase_invoice_item-> save();

            $purchase_sales_transfer_stock_track = PurchaseSalesTransferStockTrack::Where('imei_serial_no', $request->imei_serial_no)->Where('location_id', $location_id)->first();
            if ($purchase_sales_transfer_stock_track) {
                return redirect()->back()->withErrors("IMEI Already Exist In This Location");
            }else{
                $purchase_sales_transfer_stock_track = new PurchaseSalesTransferStockTrack();
                $purchase_sales_transfer_stock_track-> location_id = $location_id;
                $purchase_sales_transfer_stock_track-> warehouse_id = $request->warehouse_id;
                $purchase_sales_transfer_stock_track-> item_name_id = $request->item_name_id;
                $purchase_sales_transfer_stock_track-> item_unit_id = $request->item_unit;
                $purchase_sales_transfer_stock_track-> description = $request->description;
                // $purchase_sales_transfer_stock_track-> color = $request->color;
                $purchase_sales_transfer_stock_track-> imei_serial_no = $request->imei_serial_no;
                $purchase_sales_transfer_stock_track-> quantity = $request->quantity;
                $purchase_sales_transfer_stock_track-> rate_per_quantity = $request->rate_per_quantity;
                $purchase_sales_transfer_stock_track-> amount = $request->amount;
                $purchase_sales_transfer_stock_track-> save();
            }
            DB::commit();
            Session::flash('alert-success','Inventory Opening Balance Updated successfully');
            return redirect('/transactions/inventory-opening-balances');
        }catch(Exception $e) {
            DB::rollback();
            return redirect()->back()->withErrors($e);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\InventoryOpeningBalance  $inventoryOpeningBalance
     * @return \Illuminate\Http\Response
     */
    public function destroy(InventoryOpeningBalance $inventoryOpeningBalance)
    {
        //
    }
}
